/*****************************************************************************
* EMSO Copyright (C) 2004 - 2007 ALSOC, original code
* from http://www.rps.eng.br Copyright (C) 2002-2004.
* All rights reserved.
*
* EMSO is distributed under the therms of the ALSOC LICENSE as
* available at http://www.enq.ufrgs.br/alsoc.
*
*-----------------------------------------------------------------------------
* This file contains the prototypes and type definitions for implementing
* EMSO calculation objects in C or C++.
* In order to implement a Calculation Object module in C/C++ use the template
* file external_object_sample.c or external_object_sample.cpp
*-----------------------------------------------------------------------------
* Author: Rafael de P. Soares
* $Id$
*****************************************************************************/


#ifndef _EMSO_EXTERNAL_OBJECT_H
#define _EMSO_EXTERNAL_OBJECT_H

#include "emso_types.h"


#ifdef __cplusplus
extern "C" {
#endif

/**
* The external object create function prototype.
*/
typedef void (eo_create_prototype)(int_t *objectHandler,
								   int_t *retval, char *msg);
/**
* The implementation of the create function.
*/
eo_create_prototype EMSO_EXPORT eo_create_;


/**
* The external object destroy function prototype.
*/
typedef void (eo_destroy_prototype)(const int_t *objectHandler,
								   int_t *retval, char *msg);
/**
* The implementation of the destroy function.
*/
eo_destroy_prototype EMSO_EXPORT eo_destroy_;


/**
* The external object set parameter function prototype.
*/
typedef void (eo_set_parameter_prototype)(const int_t *objectHandler,
	const char *parameterName,
	const int_t *valueType,
	const int_t *valueLength,
	const real_t *values,
	const char *valuesText[],
	int_t *retval, char *msg);

/**
* The implementation of the set parameter function.
*/
eo_set_parameter_prototype EMSO_EXPORT eo_set_parameter_;


/**
* The external object check method function prototype.
*/
typedef void (eo_check_method_prototype)(const int_t *objectHandler,
	const char *methodName,
	int_t *methodID,
	int_t *numOfInputs,
	int_t *numOfOutputs,
	int_t *retval, char *msg);
/**
* The implementation of the check method function.
*/
eo_check_method_prototype EMSO_EXPORT eo_check_method_;



/**
* The external object check inputs function prototype.
*/
typedef void (eo_method_details_prototype)(const int_t *objectHandler,
	const char *methodName,
	const int_t *methodID,
	const int_t *numOfInputs,
	int_t *inputLengths,
	int_t *inputTypes,
	char *inputUnits[],
	const int_t *numOfOutputs,
	int_t *outputLengths,
	int_t *outputTypes,
	char *outputUnits[],
	int_t *hasDerivative,
	int_t *retval, char *msg);
/**
* The implementation of the check_inputs function.
*/
eo_method_details_prototype EMSO_EXPORT eo_method_details_;



/**
* The external object calc function prototype.
*/
typedef void (eo_calc_prototype)(const int_t *objectHandler,
	const char *methodName,
	const int_t *methodID,
	const int_t *numOfInputs,
	const int_t *inputLengths,
	const int_t *totalInputLength,
	const real_t *inputValues,
	const int_t *numOfOutputs,
	const int_t *outputLengths,
	const int_t *totalOutputLength,
	real_t *outputValues,
	const int_t *calculeDerivatives,
	real_t *outputDerivatives,
	int_t *retval, char *msg);
/**
* The implementation of the calc function.
*/
eo_calc_prototype EMSO_EXPORT eo_calc_;



#ifdef __cplusplus
}; //extern "C"
#endif


#endif //__EXT_OBJECT_H



